program linkok;
uses dos,crt;
var


PortBase : word;
MemMapped : boolean;
AutoSwitch : boolean;
VToken,FToken : string;

{$I DP83905}
{$I Parse }
{$I HREAD}
{$I HPRINT}
{$I 8259}

procedure BumOut(s: string);
begin
  writeln;
  writeln(chr(7),s);
  halt(2);
end;

procedure Usage;
begin

  writeln;
  writeln('USAGE:    LINKOK PORT [AUTO] ');
  writeln;
  writeln('          Where PORT is the HEX 4I29 base address:');
  writeln('          240,280,2C0,300,320,340,360 - use ? or S for Search');
  writeln('          If AUTO is supplied, switch to 10BaseT if Link OK');
  writeln;
  writeln('FUNCTION: Checks 10BaseT link status, ');
  writeln('          optionally switches to 10BaseT if OK');
  halt(2);
end;

procedure CheckLink;
var
link,old,new,data,funcidx : byte;
theport : word;
begin
  funcidx := 1;
  while (TokenPile[funcidx].FuncToken <> 'CableType') do funcidx := funcidx +1;
  with TokenPile[funcidx] do
  begin
    theport := PortBase + RegBOffset;
    if MemMapped then
    begin
      theport := theport + MemMapRegOffset;
    end;
    DisableInterrupts;
    data := 0;            { for 10BaseT };
    old := port[theport];
    new := (old and (FieldMask xor $FF) and GdLinkMask) or data;
    port[theport] := new;
    { 10BaseT now enabled }
    EnableInterrupts;
    delay(500);
    link := port[theport] and (GdLinkMask xor $FF);
    if link <> 0 then
    begin
      writeln('Link status OK');
      if AutoSwitch then
      begin
        writeln('10BaseT enabled');
        halt(0);
      end;
    end;  { link good }
    DisableInterrupts;
    new := port[theport];
    port[theport] := old and GdLinkMask;
    EnableInterrupts;
    if link = 0 then
    begin
      writeln('10BaseT link is down');
      if AutoSwitch and ((old and FieldMask) <> 0) then writeln('Restoring old cabletype');
      halt(2);
    end;
  end;
  halt(0);
end;

function ValidPort(p: word) : boolean;
var
funcidx,valuidx : byte;
theport : word;
begin
  ValidPort := false;
  for funcidx := 1 to MaxTokens do
  begin
    if TokenPile[funcidx].FuncToken = IOAddressName then
    begin
      with TokenPile[funcidx] do
      begin
        for valuidx := 1 to Values do
        begin
          if HexWordRead(ValuePile[valuidx].ValueToken,theport) then
          begin
            if theport = p then ValidPort := true;
          end;
        end;
      end;
    end;
  end;
end;

function AtlanticThere(p: word) : boolean;
var
olda,oldb,testa,funcidx,valuidx : byte;
theport : word;
begin
  AtlanticThere := false;
  olda := port[p + RegAOffset];
  for funcidx := 1 to MaxTokens do
  begin
    if TokenPile[funcidx].FuncToken = IOAddressName then
    begin
      with TokenPile[funcidx] do
      begin
        for valuidx := 1 to Values do
        begin
          if HexWordRead(ValuePile[valuidx].ValueToken,theport) then
          begin
            if theport = (p and $3E0) then
            {                   ^^^^^^^^^^^^ this particular kludgery }
            { is so memmapped (+10H) and io mapped places look the same }
            begin
              if (olda and FieldMask) = ValuePile[valuidx].ValueByte then
              { port select field of config reg a matches our port! }
              { may have a live one, so now we get nasty and do some writes }
              begin
                DisableInterrupts;
                testa := port[p + RegAOffset];
                { we leave everything alone except isel bits }
                testa := (olda and $C7) or $28;
                port[p + RegAOffset] := testa;
                if (port[p + RegAOffset] and $38) = $28 then
                begin
                  { we leave every thing alone except memio bit }
                  testa := port[p + RegAOffset];
                  testa := (olda and $C7) or $10;
                  port[p + RegAOffset] := testa;
                  if (port[p + RegAOffset] and $38) = $10 then
                  begin
                    { at this point we have succeeded in a simple }
                    { read write test of config reg a }
                    { now see if reg a is really reg a }
                    testa := port[p + RegAOffset];
                    oldb := port[p + RegBOffset];
                    testa := (olda and $C7) or $28;
                    { this write should fail if really atlantic }
                    port[p + RegAOffset] := testa;
                    { if the write fails, we should have our old value ! }
                    if (port[p + RegAOffset] and $38) = $10 then
                    begin
                      { Hurray! Hurray! looks like an AT/LANTIC }
                      AtlanticThere := true;
                    end;
                  end;
                end;
              end;
              { restore old reg a }
              testa := port[p + RegAOffset];
              port[p + RegAOffset] := olda;
              EnableInterrupts;
            end;
          end;
        end;
      end;
    end;
  end;
end;

function SearchForCard : boolean;
var
funcidx,valuidx : byte;
longport : longint;
foundit : boolean;
theport : word;
begin
  foundit := false;
  for funcidx := 1 to MaxTokens do
  begin
    if TokenPile[funcidx].FuncToken = IOAddressName then
    begin
      with TokenPile[funcidx] do
      begin
        valuidx := 0;
        while (valuidx < Values) and (not foundit) do
        begin
          valuidx := valuidx +1;
          if HexWordRead(ValuePile[valuidx].ValueToken,theport) then
          begin
            if AtlanticThere(theport) then
            begin
              PortBase := theport;
              foundit := true;
              MemMapped := false;
            end
            else if AtlanticThere(theport+MemMapRegOffset) then
            begin
              PortBase := word(longport);
              foundit := true;
              MemMapped := true;
            end;
          end;
        end;
      end;
    end;
  end;
  SearchForCard := foundit;
end;

procedure GetParms;
var
ps : string;
theport : word;
begin
  AutoSwitch := false;
  if ParamCount < 1 then Usage;
  if HexWordRead(ParamStr(1),theport) then
  begin
    PortBase := theport;
    if not ValidPort(PortBase) then
    begin
      writeln;
      HexPrint(PortBase,3);
      writeln(' is an invalid 4I29 port address');
      halt(2);
    end;
    if not AtlanticThere(PortBase) then
    begin
      MemMapped := true;
      if not AtlanticThere(PortBase+MemMapRegOffset) then
      begin
        writeln;
        write('No 4I29 card found at ');
        HexPrint(PortBase,3);
        writeln(' --- try search option',chr(7));
        halt(2);
      end;
    end;
  end
  else
  begin
    ps := paramStr(1);
    if (upcase(ps[1]) = 'S') or (ps[1] = '?') then
    begin
      if not SearchForCard then Bumout('Can''t find 4I29 card in system!');
    end
    else Usage;
  end;
  if ParamCount >1 then
  begin
    if UpString(Paramstr(2)) = 'AUTO' then AutoSwitch := true;
  end;
end;

{main}
begin
  DirectVideo := false;
  GetParms;
  CheckLink;
{  ShowFigs; }
end.
