program quickset;
uses dos;
var


PortBase : word;
AFig : byte;
BFig : byte;
CFig : byte;
MemMapped : boolean;
Immediate : boolean;
VToken,FToken : string;

{$I DP83905}
{$I Parse }
{$I HREAD}
{$I HPRINT}
{$I 8259}

procedure BumOut(s: string);
begin
  writeln;
  writeln(chr(7),s);
  halt(2);
end;

procedure Usage;
begin

  writeln;
  writeln('USAGE: QUICKSET PORT PARAMETER VALUE');
  writeln;
  writeln('       Where PORT is the HEX 4I29 base address:');
  writeln('       240,280,2C0,300,320,340,360 - use ? or S for Search');
  writeln('       PARAMETER is a 4I29 settable PARAMETER.');
  writeln('       VALUE is an appropiate value for that parameter.');
  writeln;
  writeln('FUNCTION: Temporarily changes 4I29 setup.');
  halt(2);
end;

procedure FBumout(ftoken : string);
var
funcidx : byte;
begin
  writeln;
  writeln(ftoken,' is not a valid parameter');
  writeln(chr(7));
  writeln('Valid parameter are:');
  writeln;
  for funcidx := 1 to MaxTokens do
  begin
    writeln('        ',TokenPile[funcidx].FuncToken);
  end;
  halt(2);
end;

procedure VBumout(ftoken,vtoken : string);
var
valuidx,funcidx : byte;
begin
  for funcidx := 1 to MaxTokens do
  begin
    if UpString(ftoken) = UpString(TokenPile[funcidx].FuncToken) then
    begin
      writeln;
      writeln(vtoken,' is not a valid value for ',TokenPile[funcidx].FuncToken);
      writeln(chr(7));
      writeln('Valid values for ',TokenPile[funcidx].FuncToken,' are:');
      writeln;
      with TokenPile[funcidx] do
      begin
        for valuidx := 1 to Values do
        begin
          writeln('        ',ValuePile[valuidx].ValueToken);
        end;
      end;
    end;
  end;
  halt(2);
end;

procedure WriteFig(fig : word;data,mask : byte);
var
old,new : byte;
theport : word;
bport : word;
begin
  theport := PortBase + fig;
  if MemMapped then
  begin
    theport := theport + MemMapRegOffset;
  end;
  DisableInterrupts;
  old := port[theport];
  if fig = RegBOffset then
  begin
    new := (old and (mask xor $FF) and GdLinkMask) or data;
  end;
  if fig = RegAOffset then
  begin
    new := (old and (mask xor $FF)) or data;
  end;
  port[theport] := new;
  EnableInterrupts;
end;


procedure ParseTokens(ftoken,vtoken : string);
var
funcidx,valuidx : byte;
funcfound,valufound : boolean;
begin
  funcfound := false;
  for funcidx := 1 to MaxTokens do
  begin
    if UpString(ftoken) = UpString(TokenPile[funcidx].FuncToken) then
    begin
      funcfound := true;
      with TokenPile[funcidx] do
      begin
        valufound:= false;
        for valuidx := 1 to Values do
        begin
          if length(vtoken) <> 0 then
          begin
            if UpString(vtoken) = UpString(ValuePile[valuidx].ValueToken) then
            begin
              valufound := true;
              case TokenReg of
                RegA : WriteFig(RegAOffset,ValuePile[valuidx].ValueByte,FieldMask);
                RegB : WriteFig(RegBOffset,ValuePile[valuidx].ValueByte,FieldMask);
                RegC :
                begin
                  if Immediate then
                  begin
                    writeln('Warning - ',ftoken,' cannot be set in immediate mode!');
                  end
                  else  CFig := CFig or ValuePile[valuidx].ValueByte;
                end;
              end;
            end;
          end;
        end;
      end;
    end;
  end;
  if not funcfound then FBumout(ftoken);
  if not valufound then VBumout(ftoken,vtoken);
end;



function ValidPort(p: word) : boolean;
var
funcidx,valuidx : byte;
theport : word;
begin
  ValidPort := false;
  for funcidx := 1 to MaxTokens do
  begin
    if TokenPile[funcidx].FuncToken = IOAddressName then
    begin
      with TokenPile[funcidx] do
      begin
        for valuidx := 1 to Values do
        begin
          if HexWordRead(ValuePile[valuidx].ValueToken,theport) then
          begin
            if theport = p then ValidPort := true;
          end;
        end;
      end;
    end;
  end;
end;

function AtlanticThere(p: word) : boolean;
var
olda,oldb,testa,funcidx,valuidx : byte;
theport : word;
begin
  AtlanticThere := false;
  olda := port[p + RegAOffset];
  for funcidx := 1 to MaxTokens do
  begin
    if TokenPile[funcidx].FuncToken = IOAddressName then
    begin
      with TokenPile[funcidx] do
      begin
        for valuidx := 1 to Values do
        begin
          if HexWordRead(ValuePile[valuidx].ValueToken,theport) then
          begin
            if theport = (p and $3E0) then
            {                   ^^^^^^^^^^^^ this particular kludgery }
            { is so memmapped (+10H) and io mapped places look the same }
            begin
              if (olda and FieldMask) = ValuePile[valuidx].ValueByte then
              { port select field of config reg a matches our port! }
              { may have a live one, so now we get nasty and do some writes }
              begin
                DisableInterrupts;
                testa := port[p + RegAOffset];
                { we leave everything alone except isel bits }
                testa := (olda and $C7) or $28;
                port[p + RegAOffset] := testa;
                if (port[p + RegAOffset] and $38) = $28 then
                begin
                  { we leave every thing alone except memio bit }
                  testa := port[p + RegAOffset];
                  testa := (olda and $C7) or $10;
                  port[p + RegAOffset] := testa;
                  if (port[p + RegAOffset] and $38) = $10 then
                  begin
                    { at this point we have succeeded in a simple }
                    { read write test of config reg a }
                    { now see if reg a is really reg a }
                    testa := port[p + RegAOffset];
                    oldb := port[p + RegBOffset];
                    testa := (olda and $C7) or $28;
                    { this write should fail if really atlantic }
                    port[p + RegAOffset] := testa;
                    { if the write fails, we should have our old value ! }
                    if (port[p + RegAOffset] and $38) = $10 then
                    begin
                      { Hurray! Hurray! looks like an AT/LANTIC }
                      AtlanticThere := true;
                    end;
                  end;
                end;
              end;
              { restore old reg a }
              testa := port[p + RegAOffset];
              port[p + RegAOffset] := olda;
              EnableInterrupts;
            end;
          end;
        end;
      end;
    end;
  end;
end;

function SearchForCard : boolean;
var
funcidx,valuidx : byte;
longport : longint;
foundit : boolean;
theport : word;
begin
  foundit := false;
  for funcidx := 1 to MaxTokens do
  begin
    if TokenPile[funcidx].FuncToken = IOAddressName then
    begin
      with TokenPile[funcidx] do
      begin
        valuidx := 0;
        while (valuidx < Values) and (not foundit) do
        begin
          valuidx := valuidx +1;
          if HexWordRead(ValuePile[valuidx].ValueToken,theport) then
          begin
            if AtlanticThere(theport) then
            begin
              PortBase := theport;
              foundit := true;
              MemMapped := false;
            end
            else if AtlanticThere(theport+MemMapRegOffset) then
            begin
              PortBase := word(longport);
              foundit := true;
              MemMapped := true;
            end;
          end;
        end;
      end;
    end;
  end;
  SearchForCard := foundit;
end;

procedure GetParms;
var
ps : string;
theport : word;
begin
  Immediate := true;
  if ParamCount < 3 then Usage;
  if HexWordRead(ParamStr(1),theport) then
  begin
    PortBase := theport;
    if not ValidPort(PortBase) then
    begin
      writeln;
      HexPrint(PortBase,3);
      writeln(' is an invalid 4I29 port address');
      halt(2);
    end;
    if not AtlanticThere(PortBase) then
    begin
      MemMapped := true;
      if not AtlanticThere(PortBase+MemMapRegOffset) then
      begin
        writeln;
        write('No 4I29 card found at ');
        HexPrint(PortBase,3);
        writeln(' --- try search option',chr(7));
        halt(2);
      end;
    end;
  end
  else
  begin
    ps := paramStr(1);
    if (upcase(ps[1]) = 'S') or (ps[1] = '?') then
    begin
      if not SearchForCard then Bumout('Can''t find 4I29 card in system!');
    end
    else Usage;
  end;
  FToken := ParamStr(2);
  VToken := ParamStr(3);
end;

procedure ShowFigs;
begin
  writeln;
  writeln('Configuration parameters are:');
  write('A register = ');
  HexPrint(AFig,2);
  writeln;
  write('B register = ');
  HexPrint(BFig,2);
  writeln;
  write('C register = ');
  HexPrint(CFig,2);
  writeln;
end;


{main}
begin
  GetParms;
  ParseTokens(FToken,VToken);
{  ShowFigs; }
end.
