program show4I29;
uses dos;
var

PortBase : word;
AFig : byte;
BFig : byte;
MemMapped : boolean;
Debug : boolean;
EAddressLow  : word;
EAddressMid  : word;
EAddressHigh : word;

{$I DP83905}
{$I HREAD}
{$I HPRINT}
{$I 8259}

procedure BumOut(s: string);
begin
  writeln;
  writeln(chr(7),s);
  halt(2);
end;

procedure Usage;
begin

  writeln;
  writeln('USAGE: SHOW4I29 PORT');
  writeln;
  writeln('    Where PORT is the HEX 4I29 base address:');
  writeln('    240,280,2C0,300,320,340,360  ? or S for Search');
  halt(2);
end;

procedure ShowFigs;
var
funcidx,valuidx,tdat : byte;
valufound : boolean;
begin
  writeln;
  for funcidx := 1 to MaxTokens do
  begin
    case TokenPile[funcidx].TokenReg of
      RegA:
      begin
        with TokenPile[funcidx] do
        begin
          write(FuncToken,' = ');
          tdat := AFig and FieldMask;
          valufound := false;
          for valuidx := 1 to Values do
          begin
            if tdat = ValuePile[valuidx].ValueByte then
            begin
              writeln(ValuePile[valuidx].ValueToken);
              valufound := true;
            end;
          end;
          if valufound = false then writeln('Unknown');
        end;
      end;
      RegB:
      begin
        with TokenPile[funcidx] do
        begin
          write(FuncToken,' = ');
          tdat := BFig and FieldMask;
          valufound := false;
          for valuidx := 1 to Values do
          begin
            if tdat = ValuePile[valuidx].ValueByte then
            begin
              writeln(ValuePile[valuidx].ValueToken);
              valufound := true;
            end;
          end;
          if valufound = false then writeln('Unknown');
        end;
      end;
    end;
  end;
  write('Ethernet Address = ');
  HexPrint(EAddressHigh,4);
  HexPrint(EAddressMid,4);
  HexPrint(EAddressLow,4);
  writeln;
end;

procedure ReadFigs;
begin
  DisableInterrupts;
  { note! depends on ps0 and ps1 being 0! }
  if MemMapped then
  begin
    AFig := port[PortBase + RegAOffset + MemMapRegOffset];
    BFig := port[PortBase + RegBOffset + MemMapRegOffset];

    EAddressHigh := word(port[PortBase + $08]) * $0100;
    EAddressHigh := EAddressHigh + word(port[PortBase + $09]);
    EAddressMid  := word(port[PortBase + $0A]) * $0100;
    EAddressMid  := EAddressMid + word(port[PortBase + $0B]);
    EAddressLow  := word(port[PortBase + $0C]) * $0100;
    EAddressLow  := EAddressLow + word(port[PortBase + $0D]);

  end
  else
  begin
    BFig := port[PortBase + RegBOffset];
    AFig := port[PortBase + RegAOffset];
    AFig := AFig or $80;
    { Temporarily set into shared memory mode to simplfy reading EAD }
    port[PortBase + RegAOffset] := AFig;

    EAddressHigh := word(port[PortBase + $08]) * $0100;
    EAddressHigh := EAddressHigh + word(port[PortBase + $09]);
    EAddressMid  := word(port[PortBase + $0A]) * $0100;
    EAddressMid  := EAddressMid + word(port[PortBase + $0B]);
    EAddressLow  := word(port[PortBase + $0C]) * $0100;
    EAddressLow  := EAddressLow + word(port[PortBase + $0D]);

    { restore I/O mode }
    AFig := port[PortBase + RegAOffset + MemMapRegOffset] and $7F;
    port[PortBase + RegAOffset + MemMapRegOffset] := AFig;
  end;
  EnableInterrupts;
end;

function ValidPort(p: word) : boolean;
var
funcidx,valuidx : byte;
theport : word;
begin
  ValidPort := false;
  for funcidx := 1 to MaxTokens do
  begin
    if TokenPile[funcidx].FuncToken = IOAddressName then
    begin
      with TokenPile[funcidx] do
      begin
        for valuidx := 1 to Values do
        begin
          if HexWordRead(ValuePile[valuidx].ValueToken,theport) then
          begin
            if theport = p then ValidPort := true;
          end;
        end;
      end;
    end;
  end;
end;

function AtlanticThere(p: word) : boolean;
var
olda,oldb,testa,funcidx,valuidx : byte;
theport : word;
begin
  if Debug then writeln('Checking for AT/LANTIC');
  AtlanticThere := false;
  olda := port[p + RegAOffset];
  for funcidx := 1 to MaxTokens do
  begin
    if TokenPile[funcidx].FuncToken = IOAddressName then
    begin
      with TokenPile[funcidx] do
      begin
        for valuidx := 1 to Values do
        begin
          if HexWordRead(ValuePile[valuidx].ValueToken,theport) then
          begin
            if theport = (p and $3E0) then
            begin
              if (olda and FieldMask) = ValuePile[valuidx].ValueByte then
              { port select field of config reg a matches our port! }
              { may have a live one, so now we get nasty and do some writes }
              if Debug then write('a');
              begin
                DisableInterrupts;
                testa := port[p + RegAOffset];
                { we leave everything alone except isel bits }
                testa := (olda and $C7) or $28;
                port[p + RegAOffset] := testa;
                if (port[p + RegAOffset] and $38) = $28 then
                begin
                  if Debug then write('b');
                  { we leave every thing alone except memio bit }
                  testa := port[p + RegAOffset];
                  testa := (olda and $C7) or $10;
                  port[p + RegAOffset] := testa;
                  if (port[p + RegAOffset] and $38) = $10 then
                  begin
                    if Debug then write('c');
                    { at this point we have succeeded in a simple }
                    { read write test of config reg a }
                    { now see if reg a is really reg a }
                    testa := port[p + RegAOffset];
                    oldb := port[p + RegBOffset];
                    testa := (olda and $C7) or $28;
                    { this write should fail if really atlantic }
                    port[p + RegAOffset] := testa;
                    if (port[p + RegAOffset] and $38) = $10 then
                    begin
                      if Debug then writeln('d');
                      { Hurray! Hurray! looks like an AT/LANTIC }
                      AtlanticThere := true;
                    end;
                  end;
                end;
              end;
              { restore old reg a }
              testa := port[p + RegAOffset];
              port[p + RegAOffset] := olda;
              EnableInterrupts;
            end;
          end;
        end;
      end;
    end;
  end;
end;

function SearchForCard : boolean;
var
funcidx,valuidx : byte;
theport : word;
foundit : boolean;
begin
  foundit := false;
  for funcidx := 1 to MaxTokens do
  begin
    if TokenPile[funcidx].FuncToken = IOAddressName then
    begin
      with TokenPile[funcidx] do
      begin
        valuidx := 0;
        while (valuidx < Values) and (not foundit) do
        begin
          valuidx := valuidx +1;
          if HexwordRead(ValuePile[valuidx].ValueToken,theport) then
          begin
            if AtlanticThere(theport) then
            begin
              PortBase := theport;
              foundit := true;
              MemMapped := false;
            end
            else
            begin
              if AtlanticThere(theport+MemMapRegOffset) then
              begin
                PortBase := theport;
                foundit := true;
                MemMapped := true;
              end;
            end;
          end;
        end;
      end;
    end;
  end;
  SearchForCard := foundit;
end;

procedure GetParms;
var
ps : string;
theport : word;
begin
  if ParamCount < 1 then Usage;
  if ParamCount = 2 then
  begin
    ps := paramStr(2);
    if (upcase(ps[1]) = 'D') then Debug := true else Debug := false;
  end;
  if HexwordRead(ParamStr(1),theport) then
  begin
    PortBase := theport;
    if not ValidPort(PortBase) then
    begin
      writeln;
      HexPrint(PortBase,3);
      writeln(' is an invalid 4I29 port address');
      halt(2);
    end;
    if not AtlanticThere(PortBase) then
    begin
      MemMapped := true;
      if not AtlanticThere(PortBase+MemMapRegOffset) then
      begin
        writeln;
        write('No 4I29 card found at ');
        HexPrint(PortBase,3);
        writeln(' --- try search option',chr(7));
        halt(2);
      end;
    end;
  end
  else
  begin
    ps := paramStr(1);
    if (upcase(ps[1]) = 'S') or (ps[1] = '?') then
    begin
      if not SearchForCard then Bumout('Can''t find 4I29 card in system!');
    end
    else Usage;
  end;
end;

{main}
begin
  GetParms;
  ReadFigs;
  ShowFigs;
end.
